#include "status_ui.hpp"
#include "components.hpp"
#include "color.hpp"
#include "guecs.hpp"
#include "rand.hpp"

namespace gui {
  using namespace guecs;

  StatusUI::StatusUI(GameLevel level) :
    $level(level)
  {
    $gui.position(STATUS_UI_X, STATUS_UI_Y, STATUS_UI_WIDTH, STATUS_UI_HEIGHT);
    $gui.layout(
        "[button1 | button2 | button3]"
        "[button4 | button5 | button6]"
        "[button7 | button8 | button9]"
        "[*%(100,300)log_view]"
        "[_]"
        "[_]");
  }

  void StatusUI::render() {
    auto& world = $gui.world();

    std::vector<std::string> fake_items{
      "cinqueda", "healing_potion_small",
      "torch_crappy", "barrel_small"};

    for(auto& [name, cell] : $gui.cells()) {
      if(name == "log_view") {
        $log_to = $gui.entity("log_view");
        world.set<Rectangle>($log_to, {});
        world.set<Textual>($log_to, {"Welcome to the Game!", 20});
      } else {
        size_t selected_item = Random::uniform<size_t>(0, fake_items.size() - 1);
        fmt::println("fake items {} but size {}", selected_item, fake_items.size());
        auto& fake_item = fake_items[selected_item];

        auto button = $gui.entity(name);
        world.set<Rectangle>(button, {});
        world.set<Sprite>(button, {fake_item});
        world.set<Clickable>(button,
            guecs::make_action(*$level.world, Events::GUI::NOOP));
      }
    }

    $gui.init();
  }

  void StatusUI::draw(sf::RenderWindow &window) {
    auto &world = $gui.world();
    auto &text = world.get<Textual>($log_to);
    std::string log;
    for(auto msg : $messages) {
      log += msg + "\n";
    }
    text.update(log);
    $gui.render(window);
  }

  void StatusUI::log(std::string msg) {
    $messages.push_front(msg);
    if($messages.size() > MAX_LOG_MESSAGES) {
      $messages.pop_back();
    }
  }
}
